@@ -1,10 +1,10 @@
 class Config():
     def __init__(self):
         self.config_dict = {'backend': 'matplotlib', # May support more backend in the future
-                            'inline': False, # Set it to true if using jupyter notebook
                             'image_mean': None,
                             'image_std': None,
                             'color_mode': 'rgb',
+                            'show_rich_info': True,
                             } 
 
     def set(self, key, value):
@@ -15,13 +15,6 @@ class Config():
         return self.config_dict.get(key, None)
 
 config = Config()
-
-
-def use_inline(value: bool):
-    if value:
-        config.set('inline', True)
-    else:
-        config.set('inline', False)
         
 def set_image_mean(mean: list):
     assert len(mean)==3
@@ -33,4 +26,10 @@ def set_image_std(mean: list):
     
 def set_color_mode(value='rgb'):
     assert value in ['rgb', 'bgr']
-    config.set('color_mode', value)
\ No newline at end of file
+    config.set('color_mode', value)
+    
+def show_rich_info(flag: bool):
+    if flag:
+        config.set('show_rich_info', True)
+    else:
+        config.set('show_rich_info', False)
\ No newline at end of file
