@@ -6,8 +6,9 @@ import warnings
 from .visualization import vis_image, vis_grayscale, vis_categorical_mask, vis_flow, display_plt, animate_plt
 from .utils import isinteger, calculate_grid_layout, tensor_to_array
 
+logging.basicConfig(level=logging.INFO)
+
 logger = logging.getLogger('TorchShow')
-logger.setLevel(logging.INFO)
 
 vis_func_dict = dict(image=vis_image,
                      grayscale=vis_grayscale,
@@ -88,8 +89,8 @@ def show(x, display=True, **kwargs):
     for row in vis_list: 
         list_per_row = []
         for img in row:
-            vis, plot_cfg = visualize(img, **kwargs)
-            list_per_row.append((vis, plot_cfg))
+            vis = visualize(img, **kwargs)
+            list_per_row.append(vis)
         plot_list.append(list_per_row)
     
     if display:
@@ -104,6 +105,7 @@ def show_video(x, display=True, **kwargs):
     if isinstance(x, (np.ndarray)):
         x = x.copy()
         assert x.ndim in [3,4], "only support 3D array (N, H, W) or 4D array (N, C, H, W) in video mode"
+        print(x.shape)
         video_list = [[x]] # for a single video, make it [[vid]]
         
     elif isinstance(x, list):
@@ -135,11 +137,11 @@ def show_video(x, display=True, **kwargs):
             for video in row:
                 if t < len(video):
                     img = video[t]
-                    vis, plot_cfg = visualize(img, **kwargs)
+                    vis = visualize(img, **kwargs)
                 else:
-                    vis, plot_cfg = (None, None)
+                    vis = None
 
-                frames_at_t_per_row.append((vis, plot_cfg)) # 
+                frames_at_t_per_row.append(vis) # 
             frames_at_t.append(frames_at_t_per_row) # 
         video_vis_list.append(frames_at_t)
         
@@ -182,14 +184,17 @@ def infer_mode(x):
     ndim = len(shape)
     if shape[-1] == 3:
         mode = 'image'
-    if shape[-1] == 2:
+    elif shape[-1] == 2:
         mode = 'flow'
-    if shape[-1] == 1:
+    elif shape[-1] == 1:
         if (x.min() >= 0) and (x.max() <= 1):
             mode = 'grayscale'
-        if isinteger(np.unique(x)).all(): # If values are all integer
+        elif isinteger(np.unique(x)).all(): # If values are all integer
             mode = 'categorical_mask'
         else:
             mode = 'grayscale'
+    else:
+        raise NotImplementedError("Does support auto infer for shape {} .".format(shape))
+    logger.debug("Auto Infer: {}".format(mode))
     return mode
 
